/*
** LBreak v1.1.2
**
** Break lines at whitespace
**
** Copyright (c) 12.8.93 by Andreas Ley <andreas.ley@kit.edu>
**
** Permission to use, copy, modify, and distribute this software for any
** purpose and without fee is hereby granted, provided that the above
** copyright notice appears in all copies. This software is provided "as is"
** and without any express or implied warranties.
**
** This program has been tested on a HP9000/720 with HP-UX A.08.07
** In this environment, neither lint -u nor gcc -Wall produce any messages.
** If you encounter any errors or need to make any changes to port it
** to another platform, please contact me.
**
** Version history
**
** Version 1.1.2 - 30.5.2006
**	Added -n option
**
** Version 1.1.1 - 24.10.95
**	Increased buffer size
**
** Version 1.1 - 28.1.95
**	Evaluate environment variable COLUMNS
**	Added indentation
**
** Version 1.0 - 12.8.93
**	Initial version
*/

static char copyright[] = "@(#)Copyright (C) 1993-2006 by Andreas Ley (andreas.ley@kit.edu)";
static char sccsid[] = "@(#)lbreak 1.1.2 - Break lines at whitespace";

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/param.h>

#define	FALSE	0		/* This is the naked Truth */
#define	TRUE	1		/* and this is the Light */

char nl[] = "\n";
char nonl[] = "";

static void lbreak(src,indent,columns,lastnl)
FILE	*src;
size_t	indent,columns;
char	*lastnl;
{
	char	line[8*1024],c,*ptr,*nptr;
	size_t	len;

	while (fgets(line,sizeof(line),src)) {
		ptr=strchr(line,'\0');
		if (*(ptr-1)=='\n')
			ptr--;
		if (*(ptr-1)=='\r')
			ptr--;
		/*
		while (ptr>line && *(ptr-1)==' ')
			ptr--;
		*/
		*ptr='\0';
		ptr=line;
		len=columns;
		while (strlen(ptr)>len) {
			c=ptr[len];
			ptr[len]='\0';
			nptr=strrchr(ptr,' ');
			if (nptr) {
				*nptr='\0';
				printf("%*s%s\n",(int)(columns-len),"",ptr);
				ptr[len]=c;
				ptr=nptr+1;
			}
			else {
				printf("%*s%s\n",(int)(columns-len),"",ptr);
				ptr[len]=c;
				ptr+=len;
			}
			len=columns-indent;
		}
		printf("%*s%s%s",(int)(columns-len),"",ptr,lastnl);
	}
}



static void usage(image)
char	*image;
{
	(void)fprintf(stderr,"Usage: %s [-n] [-i indent] [-c columns] [filename...]\n",image);
	(void)fprintf(stderr,"-n  Skip last newline\n");
	exit(1);
}


int main(argc,argv)
int	argc;
char	*argv[];
{
	int		c;
	extern char	*optarg;
	extern int	optind;
	char		error[2*MAXPATHLEN+14],*ptr,*lastnl=nl;
	size_t		indent=0,columns=80;
	FILE		*src;

	ptr=getenv("COLUMNS");
	if (ptr)
		columns=atoi(ptr);

	while ((c=getopt(argc,argv,"ni:c:vh?")) != EOF)
		switch ((char)c) {
		case 'n':
			lastnl=nonl;
			break;
		case 'i':
			indent=atoi(optarg);
			break;
		case 'c':
			columns=atoi(optarg);
			break;
		case 'v':
			(void)fprintf(stderr,"%s\n",sccsid+4);
			(void)fprintf(stderr,"%s\n",copyright+4);
			exit(0);
			/*NOTREACHED*/
			break;
		case 'h':
			(void)fprintf(stderr,"%s\n",sccsid+4);
			(void)fprintf(stderr,"%s\n",copyright+4);
		/*FALLTHROUGH*/
		case '?':
			usage(argv[0]);
		}

	if (optind<argc)
		for (;optind<argc;optind++)
			if (strcmp(argv[optind],"-"))
				if ((src=fopen(argv[optind],"r"))) {
					lbreak(src,indent,columns,lastnl);
					(void)fclose(src);
				}
				else {
					(void)sprintf(error,"%s: cannot open %s",argv[0],argv[optind]);
					perror(error);
				}
			else
				lbreak(stdin,indent,columns,lastnl);
	else
		lbreak(stdin,indent,columns,lastnl);

	return(0);
}
